@@ -12,6 +12,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import os
 import re
 import time
 
@@ -56,7 +57,7 @@ def _get_key_value(string):
 
 
 def _get_dict(lines, start_index, indentation):
-    """Recursive function for parsing hpssacli output."""
+    """Recursive function for parsing hpssacli/ssacli output."""
 
     info = {}
     current_item = None
@@ -103,9 +104,9 @@ def _get_dict(lines, start_index, indentation):
 
 
 def _convert_to_dict(stdout):
-    """Wrapper function for parsing hpssacli command.
+    """Wrapper function for parsing hpssacli/ssacli command.
 
-    This function gets the output from hpssacli command
+    This function gets the output from hpssacli/ssacli command
     and calls the recursive function _get_dict to return
     the complete dictionary containing the RAID information.
     """
@@ -116,16 +117,18 @@ def _convert_to_dict(stdout):
     return info_dict
 
 
-def _hpssacli(*args, **kwargs):
-    """Wrapper function for executing hpssacli command.
+def _ssacli(*args, **kwargs):
+    """Wrapper function for executing hpssacli/ssacli command.
 
-    :param args: args to be provided to hpssacli command
+    This function executes ssacli command if it exists, else it
+    falls back to hpssacli.
+    :param args: args to be provided to hpssacli/ssacli command
     :param kwargs: kwargs to be sent to processutils except the
         following:
         - dont_transform_to_hpssa_exception - Set to True if this
           method shouldn't transform other exceptions to hpssa
           exceptions only when hpssa controller is available. This is
-          useful when the return code from hpssacli is useful for
+          useful when the return code from hpssacli/ssacli is useful for
           analysis.
     :returns: a tuple containing the stdout and stderr after running
         the process.
@@ -140,11 +143,15 @@ def _hpssacli(*args, **kwargs):
     kwargs.pop('dont_transform_to_hpssa_exception', None)
 
     try:
-        stdout, stderr = processutils.execute("hpssacli",
-                                              *args, **kwargs)
+        if os.path.exists("/usr/sbin/ssacli"):
+            stdout, stderr = processutils.execute("ssacli",
+                                                  *args, **kwargs)
+        else:
+            stdout, stderr = processutils.execute("hpssacli",
+                                                  *args, **kwargs)
     except (OSError, processutils.ProcessExecutionError) as e:
         if 'No controllers detected' in str(e):
-            msg = ("HPSSA controller not found. Enable hpssa controller"
+            msg = ("SSA controller not found. Enable ssa controller"
                    " to continue with the desired operation")
             raise exception.HPSSAOperationError(reason=msg)
         elif not dont_transform_to_hpssa_exception:
@@ -172,10 +179,10 @@ class Server(object):
         """Gets the current RAID configuration on the server.
 
         This methods gets the current RAID configuration on the server using
-        hpssacli command and returns the output.
+        hpssacli/ssacli command and returns the output.
 
-        :returns: stdout after running the hpssacli command. The output looks
-            as follows:
+        :returns: stdout after running the hpssacli/ssacli command. The output
+            looks as follows:
 
             Smart Array P822 in Slot 2
                Bus Interface: PCI
@@ -216,10 +223,10 @@ class Server(object):
                   Interface Type: SAS
                   Drive Type: Data Drive
 
-        :raises: HPSSAOperationError, if hpssacli operation failed.
+        :raises: HPSSAOperationError, if hpssacli/ssacli operation failed.
         """
-        stdout, stderr = _hpssacli("controller", "all", "show",
-                                   "config", "detail")
+        stdout, stderr = _ssacli("controller", "all", "show",
+                                 "config", "detail")
         return stdout
 
     def refresh(self):
@@ -227,9 +234,9 @@ class Server(object):
 
         This method removes all the cache information in the server
         and it's child objects, and fetches the information again from
-        the server using hpssacli command.
+        the server using hpssacli/ssacli command.
 
-        :raises: HPSSAOperationError, if hpssacli operation failed.
+        :raises: HPSSAOperationError, if hpssacli/ssacli operation failed.
         """
         config = self._get_all_details()
 
@@ -317,7 +324,12 @@ class Controller(object):
         self.unassigned_physical_drives = []
         self.raid_arrays = []
 
-        unassigned_drives = properties.get('unassigned', {})
+        # This step is needed because of the mismatch in the data returned by
+        # hpssacli and ssacli.
+        attr = ''.join(x for x in properties
+                       if x == 'Unassigned' or x == 'unassigned')
+
+        unassigned_drives = properties.get(attr, {})
         for key, value in unassigned_drives.items():
             self.unassigned_physical_drives.append(PhysicalDrive(key,
                                                                  value,
@@ -348,20 +360,19 @@ class Controller(object):
         return None
 
     def execute_cmd(self, *args, **kwargs):
-        """Execute a given hpssacli command on the controller.
+        """Execute a given hpssacli/ssacli command on the controller.
 
         This method executes a given command on the controller.
 
         :params args: a tuple consisting of sub-commands to be appended
-            after specifying the controller in hpssacli command.
+            after specifying the controller in hpssacli/ssacli command.
         :param kwargs: kwargs to be passed to execute() in processutils
-        :raises: HPSSAOperationError, if hpssacli operation failed.
+        :raises: HPSSAOperationError, if hpssacli/ssacli operation failed.
         """
-
         slot = self.properties['Slot']
         base_cmd = ("controller", "slot=%s" % slot)
         cmd = base_cmd + args
-        return _hpssacli(*cmd, **kwargs)
+        return _ssacli(*cmd, **kwargs)
 
     def create_logical_drive(self, logical_drive_info):
         """Create a logical drive on the controller.
@@ -371,7 +382,7 @@ class Controller(object):
 
         :param logical_drive_info: a dictionary containing the details
             of the logical drive as specified in raid config.
-        :raises: HPSSAOperationError, if hpssacli operation failed.
+        :raises: HPSSAOperationError, if hpssacli/ssacli operation failed.
         """
         cmd_args = []
         if 'array' in logical_drive_info:
@@ -403,10 +414,57 @@ class Controller(object):
         """Deletes all logical drives on trh controller.
 
         This method deletes all logical drives on trh controller.
-        :raises: HPSSAOperationError, if hpssacli operation failed.
+        :raises: HPSSAOperationError, if hpssacli/ssacli operation failed.
         """
         self.execute_cmd("logicaldrive", "all", "delete", "forced")
 
+    def _get_erase_command(self, drive, pattern):
+        """Return the command arguments based on the pattern.
+
+        Erase command examples:
+        1) Sanitize: "ssacli ctrl slot=0 pd 1I:1:1 modify erase
+                      erasepattern=overwrite unrestricted=off forced"
+        2) Zeros: "ssacli ctrl slot=0 pd 1I:1:1 modify erase
+                   erasepattern=zero forced"
+
+        :param drive: A string with comma separated list of drives.
+        :param pattern: A string which defines the type of erase.
+        :returns: A list of ssacli command arguments.
+        """
+        cmd_args = []
+        cmd_args.append("pd %s" % drive)
+        cmd_args.extend(['modify', 'erase', pattern])
+
+        if pattern != 'erasepattern=zero':
+            cmd_args.append('unrestricted=off')
+
+        cmd_args.append('forced')
+        return cmd_args
+
+    def erase_devices(self, drives):
+        """Perform Erase on all the drives in the controller.
+
+        This method erases all the drives in the controller by overwriting
+        the drives with the pattern. The drives will be unavailable until
+        after successful completion or failure. The possible erase pattern
+        available for sanitize erase are 'overwrite' and 'block' to perform
+        erase on HDD and SSD respectively.
+
+        If the sanitize erase is not supported on the controller it performs
+        the disk erase by overwritting with zeros.
+
+        :param drives: A list of drive objects in the controller.
+        """
+        # TODO(aparnav): Add Sanitize erase support for SSD
+        drive = ','.join([x.id for x in drives])
+        cmd_args = self._get_erase_command(drive, 'erasepattern=overwrite')
+
+        stdout = self.execute_cmd(*cmd_args)
+        if "not supported" in str(stdout):
+            cmd_args = self._get_erase_command(drive,
+                                               'erasepattern=zero')
+            self.execute_cmd(*cmd_args)
+
 
 class RaidArray(object):
     """Class for a RAID Array.
@@ -440,7 +498,7 @@ class RaidArray(object):
     def can_accomodate(self, logical_disk):
         """Check if this RAID array can accomodate the logical disk.
 
-        This method uses hpssacli command's option to check if the
+        This method uses hpssacli/ssacli command's option to check if the
         logical disk with desired size and RAID level can be created
         on this RAID array.
 
@@ -462,7 +520,7 @@ class RaidArray(object):
             stdout, stderr = self.parent.execute_cmd(
                 *args, dont_transform_to_hpssa_exception=True)
         except processutils.ProcessExecutionError as ex:
-            # hpssacli returns error code 1 when RAID level of the
+            # hpssacli/ssacli returns error code 1 when RAID level of the
             # logical disk is not supported on the array.
             # If that's the case, just return saying the logical disk
             # cannot be accomodated in the array.
@@ -507,7 +565,7 @@ class LogicalDrive(object):
                                                         return_int=True) /
                                (1024*1024*1024)) - 1
         except ValueError:
-            msg = ("hpssacli returned unknown size '%(size)s' for logical "
+            msg = ("ssacli returned unknown size '%(size)s' for logical "
                    "disk '%(logical_disk)s' of RAID array '%(array)s' in "
                    "controller '%(controller)s'." %
                    {'size': size, 'logical_disk': self.id,
@@ -548,7 +606,7 @@ class LogicalDrive(object):
                 'volume_name': self.volume_name}
 
 
-class PhysicalDrive:
+class PhysicalDrive(object):
     """Class for PhysicalDrive object."""
 
     def __init__(self, id, properties, parent):
@@ -568,7 +626,7 @@ class PhysicalDrive:
                                                         return_int=True) /
                                (1024*1024*1024))
         except ValueError:
-            msg = ("hpssacli returned unknown size '%(size)s' for physical "
+            msg = ("ssacli returned unknown size '%(size)s' for physical "
                    "disk '%(physical_disk)s' of controller "
                    "'%(controller)s'." %
                    {'size': size, 'physical_disk': self.id,
@@ -580,6 +638,7 @@ class PhysicalDrive:
         self.disk_type = constants.get_disk_type(ssa_interface)
         self.model = self.properties.get('Model')
         self.firmware = self.properties.get('Firmware Revision')
+        self.erase_status = self.properties.get('Status')
 
     def get_physical_drive_dict(self):
         """Returns a dictionary of with the details of the physical drive."""
@@ -598,4 +657,5 @@ class PhysicalDrive:
                 'interface_type': self.interface_type,
                 'model': self.model,
                 'firmware': self.firmware,
-                'status': status}
+                'status': status,
+                'erase_status': self.erase_status}
