@@ -1,4 +1,4 @@
-# Copyright 2014 Hewlett-Packard Development Company, L.P.
+# Copyright 2017 Hewlett Packard Enterprise Development Company, L.P.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -12,24 +12,18 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-__author__ = 'HP'
+__author__ = 'HPE'
 
-import base64
-import gzip
 import hashlib
-import json
-
-import requests
-from requests.packages import urllib3
-from requests.packages.urllib3 import exceptions as urllib3_exceptions
-import six
-from six.moves.urllib import parse as urlparse
+import retrying
 
 from proliantutils import exception
 from proliantutils.ilo import common
 from proliantutils.ilo import firmware_controller
+from proliantutils.ilo import mappings
 from proliantutils.ilo import operations
 from proliantutils import log
+from proliantutils import rest
 
 """ Currently this class supports only secure boot and firmware settings
 related API's .
@@ -39,7 +33,7 @@ TODO : Add rest of the API's that exists in RIBCL. """
 DEVICE_COMMON_TO_RIS = {'NETWORK': 'Pxe',
                         'CDROM': 'Cd',
                         'HDD': 'Hdd',
-                        }
+                        'ISCSI': 'UefiTarget'}
 DEVICE_RIS_TO_COMMON = dict(
     (v, k) for (k, v) in DEVICE_COMMON_TO_RIS.items())
 
@@ -61,165 +55,22 @@ POWER_STATE = {
 CLASSCODE_FOR_GPU_DEVICES = [3]
 SUBCLASSCODE_FOR_GPU_DEVICES = [0, 1, 2, 128]
 
+MAX_RETRY_ATTEMPTS = 3  # Maximum number of attempts to be retried
+MAX_TIME_BEFORE_RETRY = 7 * 1000  # wait time in milliseconds before retry
+
 LOG = log.get_logger(__name__)
 
 
-class RISOperations(operations.IloOperations):
+class RISOperations(rest.RestConnectorBase, operations.IloOperations):
+    """iLO class for RIS interface of iLO.
 
+    Implements the class used for REST based RIS services to talk to the iLO.
+    """
     def __init__(self, host, login, password, bios_password=None,
                  cacert=None):
-        self.host = host
-        self.login = login
-        self.password = password
-        self.bios_password = bios_password
-        # Message registry support
-        self.message_registries = {}
-        self.cacert = cacert
-
-        # By default, requests logs following message if verify=False
-        #   InsecureRequestWarning: Unverified HTTPS request is
-        #   being made. Adding certificate verification is strongly advised.
-        # Just disable the warning if user intentionally did this.
-        if self.cacert is None:
-            urllib3.disable_warnings(urllib3_exceptions.InsecureRequestWarning)
-
-    def _rest_op(self, operation, suburi, request_headers, request_body):
-        """Generic REST Operation handler."""
-
-        url = urlparse.urlparse('https://' + self.host + suburi)
-        # Used for logging on redirection error.
-        start_url = url.geturl()
-
-        LOG.debug(self._("%(operation)s %(url)s "
-                         "with headers '%(header)s' and body '%(body)s'"),
-                  {'operation': operation, 'url': start_url,
-                   'header': request_headers,
-                   'body': request_body})
-
-        if request_headers is None:
-            request_headers = {}
-
-        # Use self.login/self.password and Basic Auth
-        if self.login is not None and self.password is not None:
-            auth_data = self.login + ":" + self.password
-            hr = "BASIC " + base64.b64encode(
-                auth_data.encode('ascii')).decode("utf-8")
-            request_headers['Authorization'] = hr
-
-        redir_count = 5
-        while redir_count:
-            kwargs = {'headers': request_headers,
-                      'data': json.dumps(request_body)}
-            if self.cacert is not None:
-                kwargs['verify'] = self.cacert
-            else:
-                kwargs['verify'] = False
-
-            request_method = getattr(requests, operation.lower())
-            response = None
-            try:
-                response = request_method(url.geturl(), **kwargs)
-            except Exception as e:
-                LOG.debug(self._("Unable to connect to iLO. %s"), e)
-                raise exception.IloConnectionError(e)
-
-            # NOTE:Do not assume every HTTP operation will return a JSON body.
-            # For example, ExtendedError structures are only required for
-            # HTTP 400 errors and are optional elsewhere as they are mostly
-            # redundant for many of the other HTTP status code. In particular,
-            # 200 OK responses should not have to return any body.
-
-            # NOTE:  this makes sure the headers names are all lower cases
-            # because HTTP says they are case insensitive
-            # Follow HTTP redirect
-            if response.status_code == 301 and 'location' in response.headers:
-                url = urlparse.urlparse(response.headers['location'])
-                redir_count -= 1
-                LOG.debug(self._("Request redirected to %s."), url.geturl())
-            else:
-                break
-        else:
-            # Redirected for 5th time. Throw error
-            msg = (self._("URL Redirected 5 times continuously. "
-                          "URL used: %(start_url)s") %
-                   {'start_url': start_url})
-            LOG.debug(msg)
-            raise exception.IloConnectionError(msg)
-
-        response_body = {}
-        if response.text:
-            try:
-                response_body = json.loads(response.text)
-            except (TypeError, ValueError):
-                # if it doesn't decode as json
-                # NOTE:  resources may return gzipped content
-                # try to decode as gzip (we should check the headers for
-                # Content-Encoding=gzip)
-                # NOTE: json.loads on python3 raises TypeError when
-                # response.text is gzipped one.
-                try:
-                    gzipper = gzip.GzipFile(
-                        fileobj=six.BytesIO(response.text))
-                    LOG.debug(self._("Received compressed response "
-                                     "for url %(url)s."),
-                              {'url': url.geturl()})
-                    uncompressed_string = gzipper.read().decode('UTF-8')
-                    response_body = json.loads(uncompressed_string)
-                except Exception as e:
-                    LOG.debug(self._("Got invalid response "
-                                     "'%(response)s' for url %(url)s."),
-                              {'url': url.geturl(),
-                               'response': response.text})
-                    raise exception.IloError(e)
-
-        LOG.debug(self._("Received response %(response)s for url %(url)s."),
-                  {'url': url.geturl(), 'response': response_body})
-        return response.status_code, response.headers, response_body
-
-    def _rest_get(self, suburi, request_headers=None):
-        """REST GET operation.
-
-        HTTP response codes could be 500, 404 etc.
-        """
-        return self._rest_op('GET', suburi, request_headers, None)
-
-    def _rest_patch(self, suburi, request_headers, request_body):
-        """REST PATCH operation.
-
-        HTTP response codes could be 500, 404, 202 etc.
-        """
-        if not isinstance(request_headers, dict):
-            request_headers = {}
-        request_headers['Content-Type'] = 'application/json'
-        return self._rest_op('PATCH', suburi, request_headers, request_body)
-
-    def _rest_put(self, suburi, request_headers, request_body):
-        """REST PUT operation.
-
-        HTTP response codes could be 500, 404, 202 etc.
-        """
-        if not isinstance(request_headers, dict):
-            request_headers = {}
-        request_headers['Content-Type'] = 'application/json'
-        return self._rest_op('PUT', suburi, request_headers, request_body)
-
-    def _rest_post(self, suburi, request_headers, request_body):
-        """REST POST operation.
-
-        The response body after the operation could be the new resource, or
-        ExtendedError, or it could be empty.
-        """
-        if not isinstance(request_headers, dict):
-            request_headers = {}
-        request_headers['Content-Type'] = 'application/json'
-        return self._rest_op('POST', suburi, request_headers, request_body)
-
-    def _rest_delete(self, suburi, request_headers):
-        """REST DELETE operation.
-
-        HTTP response codes could be 500, 404 etc.
-        """
-        return self._rest_op('DELETE', suburi, request_headers, None)
+        super(RISOperations, self).__init__(host, login, password,
+                                            bios_password=bios_password,
+                                            cacert=cacert)
 
     def _get_collection(self, collection_uri, request_headers=None):
         """Generator function that returns collection members."""
@@ -427,6 +278,189 @@ class RISOperations(operations.IloOperations):
                     gpu_list.append(item)
         return gpu_list
 
+    def _get_storage_resource(self):
+        """Gets the SmartStorage resource if exists.
+
+        :raises: IloCommandNotSupportedError if the resource SmartStorage
+            doesn't exist.
+        :returns the tuple of SmartStorage URI, Headers and settings.
+        """
+        system = self._get_host_details()
+        if ('links' in system['Oem']['Hp'] and
+                'SmartStorage' in system['Oem']['Hp']['links']):
+            # Get the SmartStorage URI and Settings
+            storage_uri = system['Oem']['Hp']['links']['SmartStorage']['href']
+            status, headers, storage_settings = self._rest_get(storage_uri)
+
+            if status >= 300:
+                msg = self._get_extended_error(storage_settings)
+                raise exception.IloError(msg)
+
+            return headers, storage_uri, storage_settings
+        else:
+            msg = ('"links/SmartStorage" section in ComputerSystem/Oem/Hp'
+                   ' does not exist')
+            raise exception.IloCommandNotSupportedError(msg)
+
+    def _get_array_controller_resource(self):
+        """Gets the ArrayController resource if exists.
+
+        :raises: IloCommandNotSupportedError if the resource ArrayController
+            doesn't exist.
+        :returns the tuple of SmartStorage URI, Headers and settings.
+        """
+        headers, storage_uri, storage_settings = self._get_storage_resource()
+        if ('links' in storage_settings and
+                'ArrayControllers' in storage_settings['links']):
+            # Get the ArrayCOntrollers URI and Settings
+            array_uri = storage_settings['links']['ArrayControllers']['href']
+            status, headers, array_settings = self._rest_get(array_uri)
+
+            if status >= 300:
+                msg = self._get_extended_error(array_settings)
+                raise exception.IloError(msg)
+
+            return headers, array_uri, array_settings
+        else:
+            msg = ('"links/ArrayControllers" section in SmartStorage'
+                   ' does not exist')
+            raise exception.IloCommandNotSupportedError(msg)
+
+    def _create_list_of_array_controllers(self):
+        """Creates the list of Array Controller URIs.
+
+        :raises: IloCommandNotSupportedError if the ArrayControllers
+            doesnt have member "Member".
+        :returns list of ArrayControllers.
+        """
+        headers, array_uri, array_settings = (
+            self._get_array_controller_resource())
+        array_uri_links = []
+        if ('links' in array_settings and
+                'Member' in array_settings['links']):
+            array_uri_links = array_settings['links']['Member']
+        else:
+            msg = ('"links/Member" section in ArrayControllers'
+                   ' does not exist')
+            raise exception.IloCommandNotSupportedError(msg)
+        return array_uri_links
+
+    def _get_drive_type_and_speed(self):
+        """Gets the disk drive type.
+
+        :returns: A dictionary with the following keys:
+            - has_rotational: True/False. It is True if atleast one
+            rotational disk is attached.
+            - has_ssd: True/False. It is True if at least one SSD disk is
+            attached.
+            - drive_rotational_<speed>_rpm: These are set to true as
+              per the speed of the rotational disks.
+        :raises: IloCommandNotSupportedError if the PhysicalDrives resource
+            doesn't exist.
+        :raises: IloError, on an error from iLO.
+        """
+        disk_details = self._get_physical_drive_resource()
+        drive_hdd = False
+        drive_ssd = False
+        drive_details = {}
+        speed_const_list = [4800, 5400, 7200, 10000, 15000]
+        if disk_details:
+            for item in disk_details:
+                value = item['MediaType']
+                if value == "HDD":
+                    drive_hdd = True
+                    speed = item['RotationalSpeedRpm']
+                    if speed in speed_const_list:
+                        var = 'rotational_drive_' + str(speed) + '_rpm'
+                        drive_details.update({var: 'true'})
+                # Note: RIS returns value as 'SDD' for SSD drives.
+                else:
+                    drive_ssd = True
+        if drive_hdd:
+            drive_details.update({'has_rotational': 'true'})
+        if drive_ssd:
+            drive_details.update({'has_ssd': 'true'})
+        return drive_details if len(drive_details.keys()) > 0 else None
+
+    def _get_drive_resource(self, drive_name):
+        """Gets the DiskDrive resource if exists.
+
+        :param drive_name: can be either "PhysicalDrives" or
+             "LogicalDrives".
+        :returns the list of drives.
+        :raises: IloCommandNotSupportedError if the given drive resource
+            doesn't exist.
+        :raises: IloError, on an error from iLO.
+        """
+        disk_details_list = []
+        array_uri_links = self._create_list_of_array_controllers()
+        for array_link in array_uri_links:
+            _, _, member_settings = (
+                self._rest_get(array_link['href']))
+
+            if ('links' in member_settings and
+                    drive_name in member_settings['links']):
+                disk_uri = member_settings['links'][drive_name]['href']
+                headers, disk_member_uri, disk_mem = (
+                    self._rest_get(disk_uri))
+                if ('links' in disk_mem and
+                        'Member' in disk_mem['links']):
+                    for disk_link in disk_mem['links']['Member']:
+                        diskdrive_uri = disk_link['href']
+                        _, _, disk_details = (
+                            self._rest_get(diskdrive_uri))
+                        disk_details_list.append(disk_details)
+                else:
+                    msg = ('"links/Member" section in %s'
+                           ' does not exist', drive_name)
+                    raise exception.IloCommandNotSupportedError(msg)
+            else:
+                msg = ('"links/%s" section in '
+                       ' ArrayController/links/Member does not exist',
+                       drive_name)
+                raise exception.IloCommandNotSupportedError(msg)
+        if disk_details_list:
+            return disk_details_list
+
+    def _get_logical_drive_resource(self):
+        """Returns the LogicalDrives data."""
+        return self._get_drive_resource('LogicalDrives')
+
+    def _get_physical_drive_resource(self):
+        """Returns the PhysicalDrives data."""
+        return self._get_drive_resource('PhysicalDrives')
+
+    def _get_logical_raid_levels(self):
+        """Gets the different raid levels configured on a server.
+
+        :returns a dictionary of logical_raid_levels set to true.
+            Example if raid level 1+0 and 6 are configured, it returns
+            {'logical_raid_level_10': 'true',
+             'logical_raid_level_6': 'true'}
+        """
+        logical_drive_details = self._get_logical_drive_resource()
+        raid_level = {}
+        if logical_drive_details:
+            for item in logical_drive_details:
+                if 'Raid' in item:
+                    raid_level_var = "logical_raid_level_" + item['Raid']
+                    raid_level.update({raid_level_var: 'true'})
+        return raid_level if len(raid_level.keys()) > 0 else None
+
+    def _is_raid_supported(self):
+        """Get the RAID support on the server.
+
+        This method returns the raid support on the physical server. It
+        checks for the list of array controllers configured to the Smart
+        Storage. If one or more array controllers available then raid
+        is supported by the server. If none, raid is not supported.
+
+        :return: Raid support as a dictionary with true/false as its value.
+        """
+        header, uri, array_resource = self._get_array_controller_resource()
+
+        return True if array_resource['Total'] > 0 else False
+
     def _get_bios_settings_resource(self, data):
         """Get the BIOS settings resource."""
         try:
@@ -578,53 +612,38 @@ class RISOperations(operations.IloOperations):
                    ' does not exist')
             raise exception.IloCommandNotSupportedError(msg)
 
-    def _change_iscsi_settings(self, mac, iscsi_info):
-        """Change iscsi settings.
+    def _change_iscsi_settings(self, iscsi_info):
+        """Change iSCSI settings.
 
-        :param mac: MAC address of the initiator.
-        :param iscsi_info: A dictionary that contains information of iscsi
+        :param iscsi_info: A dictionary that contains information of iSCSI
                            target like target_name, lun, ip_address, port etc.
-        :raises: IloInvalidInputError, if mac provided is invalid.
         :raises: IloError, on an error from iLO.
         """
         headers, bios_uri, bios_settings = self._check_bios_resource()
-        # Get the Boot resource and Mappings resource.
+        # Get the Mappings resource.
         map_settings = self._get_bios_mappings_resource(bios_settings)
-        boot_settings = self._get_bios_boot_resource(bios_settings)
-        correlatable_id = None
-        for boot_setting in boot_settings['BootSources']:
-            if(mac in boot_setting['UEFIDevicePath']):
-                correlatable_id = boot_setting['CorrelatableID']
-                break
+        nics = []
+        for mapping in map_settings['BiosPciSettingsMappings']:
+            for subinstance in mapping['Subinstances']:
+                for association in subinstance['Associations']:
+                    if 'NicBoot' in association:
+                        nics.append(association)
 
-        if not correlatable_id:
-            msg = ('MAC provided is Invalid')
-            raise exception.IloInvalidInputError(msg)
-
-        nic = None
-        # Get the NIC for the particular mac provided.
-        for map_setting in map_settings['BiosPciSettingsMappings']:
-            sub_instances = map_setting['Subinstances']
-            if sub_instances:
-                for sub_instance in sub_instances:
-                    if(sub_instance['CorrelatableID'] ==
-                       correlatable_id):
-                        # The nic is in the format 'NicBoot1' or 'NicBoot2'
-                        nic = sub_instance['Associations'][0]
-                        break
-                if nic is not None:
-                    break
-
-        if not nic:
-            msg = ('MAC does not have any corresponding mapping')
+        if not nics:
+            msg = ('No nics found')
             raise exception.IloError(msg)
 
         iscsi_uri = self._check_iscsi_rest_patch_allowed()
-        iscsi_info['iSCSIBootAttemptName'] = nic
-        iscsi_info['iSCSINicSource'] = nic
-        iscsi_info['iSCSIBootAttemptInstance'] = 1
-        iscsi_info['iSCSIBootEnable'] = 'Enabled'
-        patch_data = {'iSCSIBootSources': [iscsi_info]}
+        # Set iSCSI info to all nics
+        iscsi_infos = []
+        for nic in nics:
+            data = iscsi_info.copy()
+            data['iSCSIBootAttemptName'] = nic
+            data['iSCSINicSource'] = nic
+            data['iSCSIBootAttemptInstance'] = nics.index(nic) + 1
+            iscsi_infos.append(data)
+
+        patch_data = {'iSCSIBootSources': iscsi_infos}
         status, headers, response = self._rest_patch(iscsi_uri,
                                                      None, patch_data)
         if status >= 300:
@@ -825,6 +844,28 @@ class RISOperations(operations.IloOperations):
         """
         self._press_pwr_btn(pushType="PressAndHold")
 
+    @retrying.retry(
+        stop_max_attempt_number=MAX_RETRY_ATTEMPTS,
+        retry_on_result=lambda state: state != 'ON',
+        wait_fixed=MAX_TIME_BEFORE_RETRY
+    )
+    def _retry_until_powered_on(self, power):
+        """This method retries power on operation.
+
+        :param: power : target power state
+        """
+        # If the system is in the same power state as
+        # requested by the user, it gives the error
+        # InvalidOperationForSystemState. To avoid this error
+        # the power state is checked before power on
+        # operation is performed.
+        status = self.get_host_power_status()
+        if (status != power):
+            self._perform_power_op(POWER_STATE[power])
+            return self.get_host_power_status()
+        else:
+            return status
+
     def set_host_power(self, power):
         """Toggle the power button of server.
 
@@ -845,8 +886,10 @@ class RISOperations(operations.IloOperations):
             LOG.debug(self._("Node is already in '%(power)s' power state."),
                       {'power': power})
             return
-
-        self._perform_power_op(POWER_STATE[power])
+        if power == 'ON' and 'Proliant BL' in self.get_product_name():
+            self._retry_until_powered_on(power)
+        else:
+            self._perform_power_op(POWER_STATE[power])
 
     def get_http_boot_url(self):
         """Request the http boot url from system in uefi boot mode.
@@ -876,15 +919,13 @@ class RISOperations(operations.IloOperations):
             msg = 'set_http_boot_url is not supported in the BIOS boot mode'
             raise exception.IloCommandNotSupportedInBiosError(msg)
 
-    def set_iscsi_boot_info(self, mac, target_name, lun, ip_address,
-                            port='3260', auth_method=None, username=None,
-                            password=None):
-        """Set iscsi details of the system in uefi boot mode.
+    def set_iscsi_info(self, target_name, lun, ip_address,
+                       port='3260', auth_method=None, username=None,
+                       password=None):
+        """Set iSCSI details of the system in UEFI boot mode.
 
-        The iSCSI initiator is identified by the MAC provided.
         The initiator system is set with the target details like
         IQN, LUN, IP, Port etc.
-        :param mac: MAC address of initiator.
         :param target_name: Target Name for iscsi.
         :param lun: logical unit number.
         :param ip_address: IP address of the target.
@@ -894,7 +935,7 @@ class RISOperations(operations.IloOperations):
         :param password: CHAP secret.
         :raises: IloError, on an error from iLO.
         :raises: IloCommandNotSupportedInBiosError, if the system is
-                 in the bios boot mode.
+                 in the BIOS boot mode.
         """
         if(self._is_boot_mode_uefi() is True):
             iscsi_info = {}
@@ -902,15 +943,73 @@ class RISOperations(operations.IloOperations):
             iscsi_info['iSCSIBootLUN'] = lun
             iscsi_info['iSCSITargetIpAddress'] = ip_address
             iscsi_info['iSCSITargetTcpPort'] = int(port)
+            iscsi_info['iSCSITargetInfoViaDHCP'] = False
+            iscsi_info['iSCSIBootEnable'] = 'Enabled'
             if (auth_method == 'CHAP'):
                 iscsi_info['iSCSIAuthenticationMethod'] = 'Chap'
                 iscsi_info['iSCSIChapUsername'] = username
                 iscsi_info['iSCSIChapSecret'] = password
-            self._change_iscsi_settings(mac.upper(), iscsi_info)
+            self._change_iscsi_settings(iscsi_info)
         else:
-            msg = 'iscsi boot is not supported in the BIOS boot mode'
+            msg = 'iSCSI boot is not supported in the BIOS boot mode'
             raise exception.IloCommandNotSupportedInBiosError(msg)
 
+    def unset_iscsi_info(self):
+        """Disable iSCSI boot option in UEFI boot mode.
+
+        :raises: IloError, on an error from iLO.
+        :raises: IloCommandNotSupportedInBiosError, if the system is
+                 in the BIOS boot mode.
+        """
+        if(self._is_boot_mode_uefi() is True):
+            iscsi_info = {'iSCSIBootEnable': 'Disabled'}
+            self._change_iscsi_settings(iscsi_info)
+        else:
+            msg = 'iSCSI boot is not supported in the BIOS boot mode'
+            raise exception.IloCommandNotSupportedInBiosError(msg)
+
+    def get_iscsi_initiator_info(self):
+        """Give iSCSI initiator information of iLO.
+
+        :returns: iSCSI initiator information.
+        :raises: IloError, on an error from iLO.
+        :raises: IloCommandNotSupportedError, if the system is
+                 in the bios boot mode.
+        """
+        headers, bios_uri, bios_settings = self._check_bios_resource()
+        if('links' in bios_settings and 'iScsi' in bios_settings['links']):
+            iscsi_uri = bios_settings['links']['iScsi']['href']
+            status, headers, iscsi_settings = self._rest_get(iscsi_uri)
+
+            if status != 200:
+                msg = self._get_extended_error(iscsi_settings)
+                raise exception.IloError(msg)
+            return iscsi_settings['iSCSIInitiatorName']
+        else:
+            msg = ('"links/iScsi" section in bios '
+                   'does not exist')
+            raise exception.IloCommandNotSupportedError(msg)
+
+    def set_iscsi_initiator_info(self, initiator_iqn):
+        """Set iSCSI initiator information in iLO.
+
+        :param initiator_iqn: Initiator iqn for iLO.
+        :raises: IloError, on an error from iLO.
+        :raises: IloCommandNotSupportedError, if the system is
+                 in the bios boot mode.
+        """
+        if(self._is_boot_mode_uefi() is True):
+            iscsi_uri = self._check_iscsi_rest_patch_allowed()
+            initiator_info = {'iSCSIInitiatorName': initiator_iqn}
+            status, headers, response = self._rest_patch(iscsi_uri,
+                                                         None, initiator_info)
+            if status >= 300:
+                msg = self._get_extended_error(response)
+                raise exception.IloError(msg)
+        else:
+            msg = 'iSCSI initiator cannot be set in the BIOS boot mode'
+            raise exception.IloCommandNotSupportedError(msg)
+
     def get_current_boot_mode(self):
         """Retrieves the current boot mode of the server.
 
@@ -962,6 +1061,25 @@ class RISOperations(operations.IloOperations):
         # Change the Boot Mode
         self._change_bios_setting(boot_properties)
 
+    def get_supported_boot_mode(self):
+        """Retrieves the supported boot mode.
+
+        :returns: any one of the following proliantutils.ilo.constants:
+
+            SUPPORTED_BOOT_MODE_LEGACY_BIOS_ONLY,
+            SUPPORTED_BOOT_MODE_UEFI_ONLY,
+            SUPPORTED_BOOT_MODE_LEGACY_BIOS_AND_UEFI
+
+        """
+        system = self._get_host_details()
+        bios_uefi_class_val = 0  # value for bios_only boot mode
+        if ('Bios' in system['Oem']['Hp'] and
+                'UefiClass' in system['Oem']['Hp']['Bios']):
+            bios_uefi_class_val = (system['Oem']['Hp']
+                                         ['Bios']['UefiClass'])
+        return mappings.GET_SUPPORTED_BOOT_MODE_RIS_MAP.get(
+            bios_uefi_class_val)
+
     def reset_ilo_credential(self, password):
         """Resets the iLO password.
 
@@ -1100,6 +1218,10 @@ class RISOperations(operations.IloOperations):
         except Exception:
             return None
 
+    def _is_sriov_enabled(self):
+        """Return sriov enabled or not"""
+        return (self._get_bios_setting('Sriov') == 'Enabled')
+
     def get_server_capabilities(self):
         """Gets server properties which can be used for scheduling
 
@@ -1116,6 +1238,32 @@ class RISOperations(operations.IloOperations):
         capabilities['rom_firmware_version'] = rom_firmware_version
         capabilities.update(self._get_ilo_firmware_version())
         capabilities.update(self._get_number_of_gpu_devices_connected())
+        drive_details = self._get_drive_type_and_speed()
+        if drive_details is not None:
+            capabilities.update(drive_details)
+        raid_details = self._get_logical_raid_levels()
+        if raid_details is not None:
+            capabilities.update(raid_details)
+        if self._is_raid_supported():
+            capabilities['hardware_supports_raid'] = 'true'
+        boot_modes = common.get_supported_boot_modes(
+            self.get_supported_boot_mode())
+        capabilities.update({
+            'boot_mode_bios': boot_modes.boot_mode_bios,
+            'boot_mode_uefi': boot_modes.boot_mode_uefi})
+        if self._get_tpm_capability():
+            capabilities['trusted_boot'] = 'true'
+        if self._get_cpu_virtualization():
+            capabilities['cpu_vt'] = 'true'
+        if self._get_nvdimm_n_status():
+            capabilities['nvdimm_n'] = 'true'
+        try:
+            self._check_iscsi_rest_patch_allowed()
+            capabilities['iscsi_boot'] = 'true'
+        except exception.IloError:
+            # If an error is raised dont populate the capability
+            # iscsi_boot
+            pass
         try:
             self.get_secure_boot_mode()
             capabilities['secure_boot'] = 'true'
@@ -1123,6 +1271,8 @@ class RISOperations(operations.IloOperations):
             # If an error is raised dont populate the capability
             # secure_boot
             pass
+        if self._is_sriov_enabled():
+            capabilities['sriov_enabled'] = 'true'
         return capabilities
 
     def activate_license(self, key):
@@ -1437,7 +1587,6 @@ class RISOperations(operations.IloOperations):
         """
         tenure = 'Once'
         new_device = device_type[0]
-
         # If it is a standard device, we need to convert in RIS convention
         if device_type[0].upper() in DEVICE_COMMON_TO_RIS:
             new_device = DEVICE_COMMON_TO_RIS[device_type[0].upper()]
@@ -1445,11 +1594,35 @@ class RISOperations(operations.IloOperations):
         if persistent:
             tenure = 'Continuous'
 
+        systems_uri = "/rest/v1/Systems/1"
+        # Need to set this option first if device is 'UefiTarget'
+        if new_device is 'UefiTarget':
+            system = self._get_host_details()
+            uefi_devices = (
+                system['Boot']['UefiTargetBootSourceOverrideSupported'])
+            iscsi_device = None
+            for device in uefi_devices:
+                if device is not None and 'iSCSI' in device:
+                    iscsi_device = device
+                    break
+
+            if iscsi_device is None:
+                msg = 'No UEFI iSCSI bootable device found'
+                raise exception.IloError(msg)
+
+            new_boot_settings = {}
+            new_boot_settings['Boot'] = {'UefiTargetBootSourceOverride':
+                                         iscsi_device}
+
+            status, headers, response = self._rest_patch(systems_uri, None,
+                                                         new_boot_settings)
+            if status >= 300:
+                msg = self._get_extended_error(response)
+                raise exception.IloError(msg)
+
         new_boot_settings = {}
         new_boot_settings['Boot'] = {'BootSourceOverrideEnabled': tenure,
                                      'BootSourceOverrideTarget': new_device}
-        systems_uri = "/rest/v1/Systems/1"
-
         status, headers, response = self._rest_patch(systems_uri, None,
                                                      new_boot_settings)
         if status >= 300:
@@ -1467,8 +1640,9 @@ class RISOperations(operations.IloOperations):
         # Check if the input is valid
         for item in device_type:
             if item.upper() not in DEVICE_COMMON_TO_RIS:
-                raise exception.IloInvalidInputError(
-                    "Invalid input. Valid devices: NETWORK, HDD or CDROM.")
+                raise exception.IloInvalidInputError("Invalid input. Valid "
+                                                     "devices: NETWORK, HDD,"
+                                                     " ISCSI or CDROM.")
 
         self._update_persistent_boot(device_type, persistent=True)
 
@@ -1604,3 +1778,45 @@ class RISOperations(operations.IloOperations):
         gpu_devices = self._get_gpu_pci_devices()
         gpu_devices_count = len(gpu_devices)
         return {'pci_gpu_devices': gpu_devices_count}
+
+    def _get_tpm_capability(self):
+        """Retrieves if server is TPM capable or not.
+
+        :returns True if TPM is Present else False
+        """
+        tpm_values = {"NotPresent": False,
+                      "PresentDisabled": True,
+                      "PresentEnabled": True}
+        try:
+            tpm_state = self._get_bios_setting('TpmState')
+        except exception.IloCommandNotSupportedError:
+            tpm_state = "NotPresent"
+        tpm_result = tpm_values[tpm_state]
+        return tpm_result
+
+    def _get_cpu_virtualization(self):
+        """get cpu virtualization status."""
+        try:
+            cpu_vt = self._get_bios_setting('ProcVirtualization')
+        except exception.IloCommandNotSupportedError:
+            return False
+        if cpu_vt == 'Enabled':
+            vt_status = True
+        else:
+            vt_status = False
+        return vt_status
+
+    def _get_nvdimm_n_status(self):
+        """Get status of NVDIMM_N.
+
+        :returns: True if NVDIMM_N is present and enabled, False otherwise.
+        """
+        try:
+            nvdimm_n_status = self._get_bios_setting('NvDimmNMemFunctionality')
+            if nvdimm_n_status == 'Enabled':
+                nvn_status = True
+            else:
+                nvn_status = False
+        except exception.IloCommandNotSupportedError:
+            nvn_status = False
+        return nvn_status
